%This script values a CCGT via the LSM algorithm, by reading data from an
%excel file and writing its output to the same file.

%Written by Vegard Medb and ystein Arvesen, June 5th 2012

global polydeg;
global numstates;
global CI;
global efficiency;
global var_cost;
global output;
global production_matrix_el_1;
global production_matrix_el_2;
global production_matrix_gas_1;
global production_matrix_gas_2;
global gammas;
global determ;
global alpha;
global beta;
global ECconst;
global R;
global garchparams;
global firstprices;
global muMV;
global deltaMV;
global gammaMV;
global alphaMV;
global betaMV;
global powerSeasonality;
global gasSeasonality;
global disc_rate;

%Define a column, or several, to read input data from. Output will be
%written to the same file automatically.

readarea = 'c3:c97';
INparameters = xlsread('Input.xlsx', '1', readarea);

%Read pre-calculated databases of 
% production_matrix_gas_1 = reshape(dlmread('production_matrix_gas_1.txt'),90,90,50,5,5);
% production_matrix_gas_2 = reshape(dlmread('production_matrix_gas_2.txt'),90,90,50,5,5);
% production_matrix_el_1 = reshape(dlmread('production_matrix_el_1.txt'),90,90,50,5,5);
% production_matrix_el_2 = reshape(dlmread('production_matrix_el_2.txt'),90,90,50,5,5);

numstates = 5;


%%%%%%%%%%%%%%FORMATTING INPUT BEFORE USE%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
numVariations = size(INparameters,2);
%MNIG distributin aprameters
alphaMV = INparameters(1,:);
betaMV = INparameters(2:4,:);
gammaMV = zeros(3,3,numVariations);
for var_p = 1:numVariations
    gammaMV(:,1,var_p) = INparameters(5:7,var_p);
    gammaMV(:,2,var_p) = INparameters(8:10,var_p);
    gammaMV(:,3,var_p) = INparameters(11:13,var_p);
end
muMV = INparameters(14:16,:);
deltaMV = INparameters(17,:);
%GARCH parameters
garchparams = zeros(3,3,numVariations);
for var_p = 1:numVariations
    garchparams(:,1,var_p) = INparameters(18:20,var_p);
    garchparams(:,2,var_p) = INparameters(21:23,var_p);
    garchparams(:,3,var_p) = INparameters(24:26,var_p);
end

R = zeros(3,3,numVariations);
for var_p = 1:numVariations
    for j = 1:3
        R(j,j,var_p) = 1;
    end
    R(2,1) = INparameters(27,var_p);
    R(1,2) = INparameters(27,var_p);
    R(3,1) = INparameters(28,var_p);
    R(1,3) = INparameters(28,var_p);
    R(2,3) = INparameters(29,var_p);
    R(3,2) = INparameters(29,var_p);
end

%First prices, used to initialize lag-dependent model
firstprices = zeros(3,3,numVariations);
for var_p = 1:numVariations
    firstprices(:,1,var_p) = INparameters(30:32,var_p);
    firstprices(:,2,var_p) = INparameters(33:35,var_p);
    firstprices(:,3,var_p) = INparameters(36:38,var_p);
end

%Cointegration parameters
beta = INparameters(39:41, :);
alpha = INparameters(42:44,:);
ECconst = INparameters(45,:);

gammas = zeros(6,3,numVariations);
for var_p = 1:numVariations
    gammas(1,1,var_p) = INparameters(46,var_p);
    gammas(2,1,var_p) = INparameters(47,var_p);
    gammas(2,2,var_p) = INparameters(48,var_p);
    gammas(2,3,var_p) = INparameters(49,var_p);
    gammas(3,3,var_p) = INparameters(50,var_p);
    gammas(4,1,var_p) = INparameters(51,var_p);
    gammas(5,2,var_p) = INparameters(52,var_p);
end
%Weekday coefficients
determ = zeros(3,6,numVariations);
for var_p = 1:numVariations
    determ(2,1,var_p) = INparameters(53, var_p);
    determ(2,6,var_p) = INparameters(54, var_p);
    determ(1,6,var_p) = INparameters(55, var_p);
end
%Coefficients of trigonometric seasonality functions
powerSeasonality = INparameters(56:59, :);
gasSeasonality = INparameters(60:64, :);

%%%%%%%%%%plant parameters %%%%%%%%%%%%%%%%
var_cost = INparameters(65,:);
output = INparameters(78:82,:);
%startgas = INparameters(66:70,:);
efficiency = INparameters(71:75,:);
CI = INparameters(81,:);
T = INparameters(82,:);
numscenarios = INparameters(83,:);
disc_rate = INparameters(84,:);
regressiontype = INparameters(85,:);
polydeg  = INparameters(86,:);
lambdaUp = INparameters(87,:);
lambdaDown = INparameters(88,:);
muUp = INparameters(89,:);
muDown = INparameters(90,:);
sigUp = INparameters(91,:);
sigDown = INparameters(92,:);
thetaMV = INparameters(93:95,:);
%Risk neutral adjustment to skewness
betaMV = betaMV + thetaMV;

%%%%%%%%%%%%%%%  Start variations %%%%%%%%%%%%%%%%%%%%

VARvalues = zeros(1,numVariations);
VARpfvalues = zeros(1,numVariations);
VARtimes = zeros(1,numVariations) ;
VARnumSwitchons = zeros(1,numVariations) ;
VARnumSwitchoffs = zeros(1,numVariations) ;
VARstdev = zeros(1,numVariations) ;

%For each column of input data, perform the valuation and write to the same
%column:

for var_p = 1:numVariations
    disp('Variation number: ');
    disp(var_p);
    disp('out of ');
    disp(numVariations);
    tic
    
    if(regressiontype(var_p) == 1)
        regressionstyle = 'twovarreg';
    elseif(regressiontype(var_p) == 2)
        regressionstyle = 'laguerre';
    else
        regressionstyle = 'poly';
    end

%Create a (T, numscenarios, 5) matrix of prices, where the 5 different 
%prices are power, gas, carbon, CSS(efficiency1), CSS(efficiency2)
prices = priceSimulation(T(var_p), numscenarios(var_p), var_p);


%%%%%%%%%%%%%Computing spark spread for two different heat rates%%%%%%%%%%
for t = 1:T(var_p)
    for m = 1:numscenarios(var_p)
        prices(t,m,4) = prices(t,m,1) - prices(t,m,2)/efficiency(4, var_p) - prices(t,m,3)*CI(var_p)/efficiency(4, var_p);
        prices(t,m,5) = prices(t,m,1) - prices(t,m,2)/efficiency(5, var_p) - prices(t,m,3)*CI(var_p)/efficiency(5, var_p);
    end
end

%priceSimulation simulates 6-day weeks. Add inn each sunday, which has the
%same price as the preceeding saturday.
numWeeks = floor(T(var_p)/6);
weekendprices = zeros(numWeeks*7, numscenarios(var_p), numstates);
for w = 1:numWeeks
    weekendprices((w-1)*7+1:w*7-1, :, :) = prices((w-1)*6+1:w*6, :, :);
    weekendprices(w*7, :, :) = weekendprices(w*7-1, :, :);
end

prices = weekendprices;
clear weekendprices;
T(var_p) = size(prices,1);

%Initializing values at terminal time. Assume boundary condition value is
%just one day of cash profit in each state.
values = zeros(T(var_p), numscenarios(var_p), numstates);
expectedvalues = zeros(T(var_p), numscenarios(var_p), numstates);
values(T(var_p),:, 5) = 24*output(5, var_p)*(prices(T(var_p), :, 5) - var_cost(var_p));
values(T(var_p),:, 4) = 24*output(4, var_p)*(prices(T(var_p), :,4) - var_cost(var_p));
states = ones(T(var_p), numscenarios(var_p), numstates);



%Computing expected values of time T
for state1 = numstates:-1:1
    
    switch regressionstyle
        
        case 'twovarreg'
            pp = prices(T(var_p)-1, :,1);  %powerprice
            fc = prices(T(var_p)-1,:,2) + CI(var_p)*prices(T(var_p)-1,:,3);  %fuelcost
            B = multivarRegress(values(T(var_p), :, state1),  pp, fc);
            expectedvalues(T(var_p), :, state1) = B(1)*ones(size(pp)) + B(2)*pp + B(3)*pp.^2 + B(4)*fc + B(5)*fc.^2 + B(6)*pp.*fc;
            
        case 'laguerre'
            css = prices( T(var_p) - 1, :, max(state1, 4))';
            
            L = zeros(size(css,1), polydeg(var_p) + 1);
            L(:,1) = ones(size(css));
            L(:,2) = L(:,1) - css;
                if(polydeg(var_p) > 1)
                    for n = 2:polydeg(var_p)
                        L(:,n+1) = (1/(n+1))*(((2*n + 1)*ones(size(css)) - css).*L(:,n) - n*L(:,n-1));
                    end
                end
           
            B = regress(values(T(var_p), :, state1)', L);
                    
            expectedvalues(T(var_p), :, state1) = B(1)*L(:,1)' + B(2)*L(:,2)';
                if(polydeg(var_p) > 1)
                    for n = 3:polydeg(var_p)+1
                        expectedvalues(t, :, state1) = expectedvalues(T(var_p), :, state1) + B(n)*L(:,n)';
                    end
                end
            
        otherwise
            [P, S, MU] = polyfit(prices(T(var_p)-1, :, max(state1,4)), values(T(var_p),:,state1), polydeg(var_p));
            expectedvalues(T(var_p), :, state1) = polyval(P, prices(T(var_p)-1, :, max(state1,4)), [], MU);
    end
   
end

%Begin solving the dynamic program, moving backwards from T-1
for t = T(var_p)-1:-1:1
    
    for state1 = numstates:-1:1
        
        for m = 1:numscenarios(var_p)
            
            %Calculating indices for pre-defined production matrices.
            el1 = floor( ( (prices(t,m,1) - var_cost(var_p)) / (prices(t,m,2) + CI(var_p) * prices(t,m,3) ) - 1)*90/4) ;
            el2 = floor( ( (prices(t+1,m,1) - var_cost(var_p)) / (prices(t+1,m,2) + CI(var_p) * prices(t+1,m,3) ) - 1)*90/4) ;
            gas = floor( ( (prices(t+1,m,2) + CI(var_p) * prices(t+1,m,3)) / (prices(t,m,2) + CI(var_p) * prices(t,m,3)) -1/2 ) * 50/3) ;
            
                    
            el1 = min(max(1, el1), 90);
            el2 = min(max(1, el2), 90);
            gas = min(max(1, gas), 50);
            

            maxvalue=-inf;
            revenue = zeros(numstates, 1);
            
            %Decide what next state is the best state based on expected
            %continuation value
            for state2 = 1:numstates
                
                revenue(state2,1) = production_matrix_el_1(el1,el2,gas,state1,state2) * (prices(t,m,1) - var_cost(var_p)) + production_matrix_el_2(el1,el2,gas,state1,state2) * (prices(t+1,m,1) - var_cost(var_p)) - production_matrix_gas_1(el1,el2,gas,state1,state2) * (prices(t,m,2) + CI(var_p) * prices(t,m,3)) - production_matrix_gas_2(el1,el2,gas,state1,state2)*(prices(t+1,m,2) + CI(var_p) * prices(t+1,m,3));
                if(mod(t,6) == 0)
                    temp_value = revenue(state2,1)+ exp(-disc_rate(var_p)/365)*values(t+1, m, state2);
                else
                    temp_value = revenue(state2,1)+ exp(-disc_rate(var_p)/365)*expectedvalues(t+1, m, state2);
                end
                
                if(temp_value > maxvalue)
                    maxvalue = temp_value;
                    nextstate = state2;
                    conditional_revenue = revenue(state2,1);
                end
                
                
            end
            
           
            states(t,m,state1) = nextstate;
            values(t,m,state1) = conditional_revenue + exp(-disc_rate(var_p)/365)*values(t+1,m,nextstate);
            
        end  
        %Compute new expected values for t, based on information in t-1 
        if(t>1)
            switch regressionstyle
                case 'twovarreg'
                    pp = prices(t-1, :,1);  %powerprice
                    fc = prices(t-1,:,2) + CI(var_p)*prices(t-1,:,3);  %fuelcost
                    B = multivarRegress(values(t, :, state1),  pp, fc);
                    expectedvalues(t, :, state1) = B(1) + B(2)*pp + B(3)*pp.^2 + B(4)*fc + B(5)*fc.^2 + B(6)*pp.*fc;
                    
                case 'laguerre'
                    css = prices( t - 1, :, max(state1, 4))';
                    
                    L = zeros(size(css,1), polydeg(var_p) + 1);
                    L(:,1) = ones(size(css));
                    L(:,2) = L(:,1) - css;
            
                    if(polydeg(var_p) > 1)
                        for n = 2:polydeg(var_p)
                            L(:,n+1) = (1/(n+1))*(((2*n + 1)*ones(size(css)) - css).*L(:,n) - n*L(:,n-1));
                        end
                    end

                    
                    B = regress(values(t, :, max(state1, 4))', L);
                    
                    expectedvalues(t, :, state1) = B(1)*L(:,1)' + B(2)*L(:,2)';
                    if(polydeg(var_p) > 1)
                        for n = 3:polydeg(var_p)+1
                            expectedvalues(t, :, state1) = expectedvalues(t, :, state1) + B(n)*L(:,n)';
                        end
                    end
            
                otherwise
                    [P, S, MU] = polyfit(prices(t-1, :, max(state1,4)), values(t,:,state1), polydeg(var_p));
                    expectedvalues(t, :, state1) = polyval(P, prices(t-1, :, max(state1,4)), [], MU);
            end
            
                  
        end
        

    end

end


%Loop is complete. Retrieve optimal state sequence    
optimalstates = zeros(T(var_p), numscenarios(var_p));
optimalstates(1,:) = 1;
for m = 1:numscenarios(var_p)
    for t = 1:T-1
        optimalstates(t+1,m) = states(t,m,optimalstates(t,m));
        
    end
end

%Compute the value of the plant assuming perfect foresight
[PFvalues, PFstates] = perfectForesight(T(var_p), numscenarios(var_p), prices, var_p);


simulationValue = mean(values(1,:,1));
perfectForesightValue = mean(PFvalues(1,:,1));

%Compute average number of switch-ons/switch-offs per scenario
numSwitchoffs = 0;
numSwitchons = 0;
for m = 1:numscenarios(var_p)
    for t = 2:T(var_p)
        if(optimalstates(t,m) > 3 && optimalstates(t-1,m) < 4)
            numSwitchons = numSwitchons + 1;
        elseif(optimalstates(t,m) < 4 && optimalstates(t-1,m) > 3)
            numSwitchoffs = numSwitchoffs + 1;
        end
    end
end

VARvalues(1,var_p) = mean(values(1,:,1));
VARpfvalues(1,var_p) = mean(PFvalues(1,:,2));
VARstdev(1,var_p) = std(values(1,:,1));
VARtimes(1,var_p) = toc;
VARnumSwitchons(1,var_p) = (numSwitchons/numscenarios(var_p));
VARnumSwitchoffs(1,var_p) = (numSwitchoffs/numscenarios(var_p));


end

%Write results to file
xlswrite('Input.xlsx', VARvalues, '1', strcat(readarea(1), '100:', readarea(4), '100'));
xlswrite('Input.xlsx', VARpfvalues, '1', strcat(readarea(1), '101:', readarea(4), '101'));
xlswrite('Input.xlsx', VARstdev, '1', strcat(readarea(1), '102:', readarea(4), '102'));
xlswrite('Input.xlsx', VARtimes, '1', strcat(readarea(1), '103:', readarea(4), '103'));
xlswrite('Input.xlsx', VARnumSwitchons, '1', strcat(readarea(1), '104:', readarea(4), '104'));
xlswrite('Input.xlsx', VARnumSwitchoffs, '1', strcat(readarea(1), '105:', readarea(4), '105'));

beep;
