function [valuePF, statesPF] = perfectForesight( T, numscenarios, prices, var_p )
%PERFECTFORESIGHTFUNCTION computes the value of a CCGT, using the LSM
%algorithm, but assuming perfect knowledge of future prices.

%Written by Vegard Medb and ystein Arvesen
%June 5th 2012
%Part of Master Thesis at NTNU, Norway


    global numstates;    
    global CI;  
    global var_cost;
    global production_matrix_el_1;
    global production_matrix_el_2;
    global production_matrix_gas_1;
    global production_matrix_gas_2;
    global disc_rate;
    
    %VALUATION ALGORITHM
    values = zeros(T, numscenarios, numstates);
    values(T,:, 5) = 24*300*prices(T, :, 5);
    values(T,:, 4) = 24*150*prices(T, :,4);
    states = ones(T, numscenarios, numstates);
    
    
    
    for t = T(var_p)-1:-1:1  
        
        for state1 = numstates:-1:1
            
            for m = 1:numscenarios
                el1 = floor( ( (prices(t,m,1) - var_cost(var_p)) / (prices(t,m,2) + CI(var_p) * prices(t,m,3) ) -1)*90/4) ;
                el2 = floor( ( (prices(t+1,m,1) - var_cost(var_p)) / (prices(t+1,m,2) + CI(var_p) * prices(t+1,m,3) ) -1)*90/4) ;
                gas = floor( ( (prices(t+1,m,2) + CI(var_p) * prices(t+1,m,3)) / (prices(t,m,2) + CI(var_p) * prices(t,m,3)) -0.5 ) * 50/3) ;

            
                el1 = min(max(1, el1), 90);
                el2 = min(max(1, el2), 90);
                gas = min(max(1, gas), 50);


                maxvalue=-inf;
                revenue = zeros(numstates, 1);
                
                for state2 = 1:numstates
                
                    revenue(state2,1) = production_matrix_el_1(el1,el2,gas,state1,state2) * (prices(t,m,1) - var_cost(var_p)) + production_matrix_el_2(el1,el2,gas,state1,state2) * (prices(t+1,m,1) - var_cost(var_p)) - production_matrix_gas_1(el1,el2,gas,state1,state2) * (prices(t,m,2) + CI(var_p) * prices(t,m,3)) - production_matrix_gas_2(el1,el2,gas,state1,state2)*(prices(t+1,m,2) + CI(var_p) * prices(t+1,m,3));
                    %Assume perfect foresight here, using 'values' and not
                    %'expected values'
                    temp_value = revenue(state2,1)+ exp(-disc_rate(var_p)/365)*values(t+1, m, state2);
                    if(temp_value > maxvalue)
                        maxvalue = temp_value;
                        nextstate = state2;
                        conditional_revenue = revenue(state2,1);
                    end
                end
            
                states(t,m,state1) = nextstate;
                values(t,m,state1) = conditional_revenue + exp(-disc_rate(var_p)/365)*values(t+1,m,nextstate);
            end  
    
        end
    
    
    end
    
    
    
    optimalstates = zeros(T, numscenarios);
    optimalstates(1,:) = 1;
    for m = 1:numscenarios
        for t = 1:T-1
            optimalstates(t+1,m) = states(t,m,optimalstates(t,m));
    
        end
    end
    
    valuePF = values;
    statesPF = optimalstates;
    
end

